diff --git a/include/sound/core.h b/include/sound/core.h
index 1f9aef0adbc9..e7c947c934b0 100644
--- a/include/sound/core.h
+++ b/include/sound/core.h
@@ -100,6 +100,7 @@ struct snd_card {
 	struct rw_semaphore controls_rwsem;	/* controls list lock */
 	rwlock_t ctl_files_rwlock;	/* ctl_files list lock */
 	int controls_count;		/* count of all controls */
+	size_t user_ctl_max_size;	/* max memory allocation for user controls */
 	size_t user_ctl_alloc_size;	// current memory allocation by user controls.
 	struct list_head controls;	/* all controls for this card */
 	struct list_head ctl_files;	/* active control files */
diff --git a/sound/core/control.c b/sound/core/control.c
index a076c08c21b6..38706261c11b 100644
--- a/sound/core/control.c
+++ b/sound/core/control.c
@@ -1269,7 +1269,7 @@ struct user_element {
 // check whether the addition (in bytes) of user ctl element may overflow the limit.
 static bool check_user_elem_overflow(struct snd_card *card, ssize_t add)
 {
-	return (ssize_t)card->user_ctl_alloc_size + add > max_user_ctl_alloc_size;
+	return (ssize_t)card->user_ctl_alloc_size + add > card->user_ctl_max_size;
 }
 
 static int snd_ctl_elem_user_info(struct snd_kcontrol *kcontrol,
@@ -2199,6 +2199,55 @@ static int snd_ctl_dev_free(struct snd_device *device)
 	return 0;
 }
 
+/*
+ * sysfs entries
+ */
+static ssize_t
+user_ctl_max_size_show(struct device *dev, struct device_attribute *attr,
+		       char *buf)
+{
+	struct snd_card *card = container_of(dev, struct snd_card, ctl_dev);
+
+	return scnprintf(buf, PAGE_SIZE, "%ld\n", card->user_ctl_max_size);
+}
+
+static ssize_t
+user_ctl_max_size_store(struct device *dev, struct device_attribute *attr,
+			const char *buf, size_t count)
+{
+	struct snd_card *card = container_of(dev, struct snd_card, ctl_dev);
+	unsigned long size;
+	int err;
+
+	err = kstrtoul(buf, 0, &size);
+	if (err)
+		return err;
+	if (size >= 32 * 1024 * 1024 || size < card->user_ctl_alloc_size)
+		return -EINVAL;
+	card->user_ctl_max_size = size;
+	return count;
+}
+
+static DEVICE_ATTR_RW(user_ctl_max_size);
+
+static ssize_t
+user_ctl_alloc_size_show(struct device *dev, struct device_attribute *attr,
+			 char *buf)
+{
+	struct snd_card *card = container_of(dev, struct snd_card, ctl_dev);
+
+	return scnprintf(buf, PAGE_SIZE, "%ld\n", card->user_ctl_alloc_size);
+}
+
+static DEVICE_ATTR_RO(user_ctl_alloc_size);
+
+static struct attribute *ctl_dev_attrs[] = {
+	&dev_attr_user_ctl_max_size.attr,
+	&dev_attr_user_ctl_alloc_size.attr,
+	NULL
+};
+ATTRIBUTE_GROUPS(ctl_dev);
+
 /*
  * create control core:
  * called from init.c
@@ -2218,7 +2267,9 @@ int snd_ctl_create(struct snd_card *card)
 		return -ENXIO;
 
 	snd_device_initialize(&card->ctl_dev, card);
+	card->ctl_dev.groups = ctl_dev_groups;
 	dev_set_name(&card->ctl_dev, "controlC%d", card->number);
+	card->user_ctl_max_size = max_user_ctl_alloc_size;
 
 	err = snd_device_new(card, SNDRV_DEV_CONTROL, card, &ops);
 	if (err < 0)
